package dev.pfaff.altargui.async;

import dev.pfaff.altargui.util.CancellationException;

import java.util.Objects;
import java.util.concurrent.CompletableFuture;
import java.util.concurrent.CompletionException;
import java.util.concurrent.ExecutionException;
import java.util.concurrent.Future;

final class FutureAsTask<V> implements Task.Cancellable<V> {
	private final Future<V> future;

	public FutureAsTask(Future<V> future) {
		this.future = Objects.requireNonNull(future);
	}

	@Override
	public V join() throws CompletionException, CancellationException, InterruptedException {
		try {
			return this.future.get();
		} catch (java.util.concurrent.CancellationException e) {
			throw new CancellationException(e.getMessage(), e);
		} catch (ExecutionException e) {
			throw new CompletionException(e.getCause());
		}
	}

	@Override
	public V joinUninterruptibly() throws CompletionException, CancellationException {
		if (this.future instanceof CompletableFuture<V> future) {
			try {
				return future.join();
			} catch (java.util.concurrent.CancellationException e) {
				throw new CancellationException(e.getMessage(), e);
			}
		} else {
			while (true) {
				try {
					return join();
				} catch (InterruptedException e) {
					// try again
				}
			}
		}
	}

	@Override
	public void cancel() {
		this.future.cancel(true);
	}
}
