package dev.pfaff.altargui.async;

import dev.pfaff.altargui.function.FallibleRunnable;

import java.util.concurrent.Callable;

import static dev.pfaff.altargui.async.Task.joinAndCancelOnInterruptIfPossible;
import static dev.pfaff.altargui.function.FallibleRunnable.asFallible;

public interface TaskExecutor {
	<V> Task<V> submit(Callable<V> callable);

	default Task<Void> submit(Runnable runnable) {
		return submitFallible(asFallible(runnable));
	}

	default Task<Void> submitFallible(FallibleRunnable runnable) {
		return submit(runnable);
	}

	default <V> V execute(Callable<V> callable) {
		return joinAndCancelOnInterruptIfPossible(submit(callable));
	}

	default void execute(Runnable runnable) {
		executeFallible(asFallible(runnable));
	}

	default void executeFallible(FallibleRunnable runnable) {
		execute(runnable);
	}
}
