package dev.pfaff.altargui.text;

import dev.pfaff.altargui.libimage.ImageData;

import java.lang.foreign.Arena;

import static java.lang.foreign.ValueLayout.JAVA_BYTE;

public final class SDFFontUtil {
	/**
	 * Only {@code 255} and {@code 0}, uniform across channels, are valid inputs for this conversion.
	 */
	public static SDF transparentAndWhiteToSDF(ImageData imageData, Arena dstArena) {
		var stride = imageData.colorType().pixelSize();
		var offset = switch (imageData.colorType()) {
			case L8 -> 0;
			case La8 -> 1;
			case Rgb8 -> 0;
			case Rgba8 -> 3;
			case L16 -> 0;
			case La16 -> 2;
			case Rgb16 -> 0;
			case Rgba16 -> 6;
			case Rgb32F -> 0;
			case Rgba32F -> 12;
		};
		int width = imageData.width();
		int height = imageData.height();
		var src = imageData.pixels();
		var dst = dstArena.allocate(Math.divideExact(src.byteSize(), stride));
		for (long i = 0; i < dst.byteSize(); i++) {
			var pixel = src.get(JAVA_BYTE, i * stride + offset);
			dst.setAtIndex(JAVA_BYTE, i, pixel);
		}
		return new SDF(dst, width, height);
	}
}
